"""
Simple, pretty ping wrapper (for *nix).

Colorizes ping results and displays as a bar graph on a log scale.
"""

from __future__ import unicode_literals

import argparse
import datetime
import sys
import time
from multiprocessing import Process
from multiprocessing import Queue

from pyng import asyncping
from pyng import color
from pyng import settings
from pyng import utils

assert sys.version_info >= (2, 7, 0)


DESCRIPTION = "Simple, pretty ping wrapper (for *nix).  \
Use `pyng` or `pyng3` to invoke.  Uninstall with `pip uninstall pyng3` or `pip3 uninstall pyng3`."
HEADER = '|'.join(
    '{:<{}}'.format(
        utils.column_to_timescale_header(i),
        settings.WIDTH,
    )
    for i in range(settings.COLUMNS)
)
SUBHEADER = '+'.join(
    '-' * settings.WIDTH for i in range(settings.COLUMNS)
)


def graph(num):
    """
    Graphs the given ping result on a log scale.
    (must be 0 <= num <= 999999)

    Returns a single output row (no color, Unicode).
    """
    if not 0 <= num <= 999999:
        print("Got ping outside range [0, 999999]")
        return

    num = int(num)
    num_str = str(num)
    digits = len(num_str)
    first_digit = int(num_str[0])

    row = [settings.BLANK * settings.WIDTH] * settings.COLUMNS
    for i in range(digits - 1):
        blocks = settings.FILL * settings.WIDTH
        row[i] = blocks

    row[digits - 1] = '{:{}<{}}'.format(
        settings.HEAD * first_digit,
        settings.BLANK,
        settings.WIDTH,
    )

    return '|'.join(row)


def parse_args():
    """Parses and saves command-line arguments."""
    parser = argparse.ArgumentParser(description=DESCRIPTION)
    parser.add_argument('host',  default='www.google.com')
    parser.add_argument('-p', '--pings', default=-1, type=int)
    return parser.parse_args()


def print_header(host, pings):
    print('')
    print(datetime.datetime.now().strftime("%D %H:%M:%S"))
    print("Pinging: {} ({})".format(host, "{} left".format(pings) if pings > 0 else "forever"))
    print(HEADER)
    print(SUBHEADER)


def print_graph(result):
    row = graph(result)
    blocks = row.split('|')
    row = '|'.join(color.color(e, i) for (i, e) in enumerate(blocks))
    print(row)


def loop(queue, host, pings):
    """Wait for ping results and display them prettily"""
    loops = 0
    while pings != 0:
        result = queue.get()

        # header, for context
        if not loops % settings.PINGS_PER_HEADER:
            print_header(host, pings)

        if result > 0:
            print_graph(result)
        else:
            print("TIMEOUT!")

        loops += 1
        pings -= 1

        # if we get a ton of results in rapid successon, we will
        # update the screen quickly (~100/sec) until we catch up
        time.sleep(0.01)


def main():
    """Get arguments, start ping, and begin loop"""
    args = parse_args()
    host = args.host
    pings = args.pings

    queue = Queue()

    ping_thread = Process(
        target=asyncping.fetch_times,
        args=(queue, host),
        kwargs={"pings": pings},
    )
    ping_thread.start()

    try:
        loop(queue, host, pings)
    except KeyboardInterrupt:
        print("Closing.")

    ping_thread.terminate()


if __name__ == '__main__':
    main()




from __future__ import unicode_literals


# change these carefully... we use a log scale, so WIDTH ranges from 0 to 9
COLUMNS = 6
WIDTH = 9

PINGS_PER_HEADER = 20
BLANK = '.'

# defines the body and right-most end of the bar graph
# for example, if FILL = "-" and HEAD = ">", you will see "----->"
FILL = '\u2588'  # U+2588 = solid block
HEAD = '\u2588'



"""
Thread to asynchronously run `ping` and yield (queue) parsed times
"""

import re
import subprocess


TIME_REGEX = r'''time=([0-9\.]+) ms'''


def fetch_times(queue, host, pings=-1):
    """
    Runs `ping` in the background, parsing and
    appending its results to a queue.
    """

    ping = subprocess.Popen(['ping', host], stdout=subprocess.PIPE)

    # this lets us set PINGS to -1 to run forever
    while pings != 0:
        line = ping.stdout.readline().decode('utf-8').strip()

        if line and not line.startswith("PING"):
            try:
                num = int(float(re.search(TIME_REGEX, line).group(1)))
            except AttributeError:
                num = -1  # probably a timeout

            queue.put(num)
            pings -= 1



"""
PyPI setup file
"""

from setuptools import setup


setup(
    name='pyng3',
    packages=['pyng'],
    version='3.0.0',
    author='Matt Cotton',
    author_email='matthewcotton.cs@gmail.com',
    url='https://github.com/MattCCS/Pyng',

    description='Simple, pretty ping wrapper (for *nix)',
    long_description=open("README.md").read(),
    classifiers=[
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
    ],

    entry_points={
        'console_scripts': [
            'pyng=pyng.main:main',
            'pyng3=pyng.main:main',
        ],
    },
)




from __future__ import unicode_literals


def _color(string, ansi):
    """Colors the given string with the given ANSI color index."""
    return "\x1b[0;{}m{}\x1b[0m".format(ansi, string)


def green(string):
    return _color(string, 32)


def yellow(string):
    return _color(string, 33)


def red(string):
    return _color(string, 31)


def black(string):
    return _color(string, 30)


def color(string, index):
    """Colors the given string based on list position (ANSI)."""
    if index in (0, 1):
        return green(string)
    elif index == 2:
        return yellow(string)
    elif index == 3:
        return red(string)
    else:
        return black(string)
